from pydispatch import dispatcher
import hashlib
import rsa
from utils import *
from packet import Packet
from diffiehellman.diffiehellman import DiffieHellman


class Node:

    def __init__(self, id):
        self.setup_keys()
        self.connect_to_signals()
        self.id = id
        self.send_ops = {
            OP.CREATED: self.get_created_packet,
            OP.EXTENDED: self.get_extended_packet
        }
        self.hop_table = {1: HopPair(prev=None, next=None)}

    def setup_keys(self):
        self.dhke = DiffieHellman(key_length=200)
        self.dhke.generate_public_key()
        self.__dh_sharedkey = None
        self.dh_pub = str(self.dhke.public_key).encode('utf8')
        (self.pubkey, self.__privkey) = rsa.newkeys(512)

    def connect_to_signals(self):
        dispatcher.connect(self.handle_extend,
                           signal=OP.EXTEND, sender=dispatcher.Any)
        dispatcher.connect(self.handle_create,
                           signal=OP.CREATE, sender=dispatcher.Any)
        dispatcher.connect(self.handle_created,
                           signal=OP.CREATED, sender=dispatcher.Any)

    def send_packet(self, receiver, op, payload=None):
        packet = self.send_ops[op](receiver, payload)
        dispatcher.send(signal=op, sender=self, packet=packet)

    def get_created_packet(self, receiver, payload=None):
        key_hash = hashlib.sha1(
            str(self.__dh_sharedkey).encode("utf-8")).hexdigest()
        msg = (self.dhke.public_key, key_hash)
        packet = Packet(src_id=self.id, op=OP.CREATED,
                        dest=receiver, payload=(msg, None))
        print("{}: Sending CREATED packet to {}".format(self.id, receiver))
        return packet

    def get_extended_packet(self, receiver, payload):
        packet = Packet(self.id, OP.EXTENDED, receiver, payload)
        print("{}: Sending EXTENDED packet to {}".format(self.id, receiver))
        return packet

    def handle_extend(self, packet):
        if packet.dest != self.id:
            return None
        print("{}: Handling EXTEND packet from {}".format(self.id, packet.src))
        # Decrypt packet with AES
        # Forward the packet accordingly
        pass

    def handle_create(self, packet):
        if packet.dest != self.id:
            return None
        print("{}: Handling CREATE packet from {}".format(self.id, packet.src))

        # Decrypt packet with RSA
        # Generate DH shared secret
        # Send CREATED packet accordingly
        pass

     def handle_created(self, packet):
        if packet.dest != self.id:
            return None
        print("{}: Handling CREATED packet from {}".format(self.id, packet.src))

        # Get public key and hash from packet
        # Encrypt the pair using AES (hint: Look at how the pair is serialized from the client class)
        # Send the packet accordingly 
        pass


    def __str__(self):
        return self.id
